from .django14 import now, utc, patterns, url, include
from .django15 import get_user_model, AUTH_USER_MODEL
from .django16 import transaction_atomic, add_preserved_filters, is_queryset_empty
from .django17 import is_installed, get_models, get_app_names, get_app_label, truncate_name, get_meta_model_name
from .python3 import smart_text

__all__ = (
    'now', 'utc',
    'patterns', 'url', 'include',
    'get_user_model', 'AUTH_USER_MODEL',
    'transaction_atomic',
    'add_preserved_filters',
    'is_queryset_empty',
    'is_installed', 'get_models', 'get_app_names', 'get_app_label', 'truncate_name', 'get_meta_model_name',
    'smart_text',
)



"""
Optional integration with django-any-Imagefield
"""
from __future__ import absolute_import
from django.db import models
from fluent_utils.django_compat import is_installed

if is_installed('any_imagefield'):
    from any_imagefield.models import AnyFileField as BaseFileField, AnyImageField as BaseImageField
else:
    BaseFileField = models.FileField
    BaseImageField = models.ImageField


# subclassing here so South or Django migrations detect a single class.
class AnyFileField(BaseFileField):
    """
    A FileField that can refer to an uploaded file.

    If *django-any-imagefield* is not installed, the filebrowser link will not be displayed.
    """
    def south_field_triple(self):
        # Masquerade as normal FileField, so the soft-dependency also exists in the migrations.
        from south.modelsinspector import introspector
        path = "{0}.{1}".format(models.FileField.__module__, models.FileField.__name__)
        args, kwargs = introspector(self)
        return (path, args, kwargs)

    def deconstruct(self):
        # For Django 1.7 migrations, masquerade as normal FileField too
        name, path, args, kwargs = super(AnyFileField, self).deconstruct()
        path = "{0}.{1}".format(models.FileField.__module__, models.FileField.__name__)
        return name, path, args, kwargs


# subclassing here so South or Django migrations detect a single class.
class AnyImageField(BaseImageField):
    """
    An ImageField that can refer to an uploaded image file.

    If *django-any-imagefield* is not installed, the filebrowser link will not be displayed.
    """
    def south_field_triple(self):
        # Masquerade as normal ImageField, so the soft-dependency also exists in the migrations.
        from south.modelsinspector import introspector
        path = "{0}.{1}".format(models.ImageField.__module__, models.ImageField.__name__)
        args, kwargs = introspector(self)
        return (path, args, kwargs)

    def deconstruct(self):
        # For Django 1.7 migrations, masquerade as normal ImageField too
        name, path, args, kwargs = super(AnyImageField, self).deconstruct()
        path = "{0}.{1}".format(models.ImageField.__module__, models.ImageField.__name__)
        return name, path, args, kwargs



try:
    # Python 3
    from django.utils.encoding import smart_text
except ImportError:
    try:
        from django.utils.encoding import smart_unicode as smart_text
    except ImportError:
        from django.forms.util import smart_unicode as smart_text


__all__ = (
    'smart_text',
)



import sys
import logging
from django.conf import settings

logger = logging.getLogger(__name__)


try:
    from django.apps import apps
except ImportError:
    from django.db.backends.util import truncate_name

    # Django 1.6 or below.
    def is_installed(appname):
        return appname in settings.INSTALLED_APPS

    from django.db.models import get_models

    def get_app_names():
        return settings.INSTALLED_APPS

    def get_app_label(module):
        # Django <= 1.6 does not know apps.
        # Figure out the app_label by looking one level up.
        # For 'django.contrib.sites.models', this would be 'sites'.
        model_module = sys.modules[module]
        return model_module.__name__.split('.')[-2]

    def get_meta_model_name(opts):
        return opts.module_name

else:
    # util was renamed to utils for consistently
    from django.db.backends.utils import truncate_name

    # Django 1.7 provides an official API, and INSTALLED_APPS may contain non-string values too.
    get_models = apps.get_models

    def is_installed(appname):
        if apps.apps_ready:
            return apps.is_installed(appname)
        else:
            logger.debug('''fluent_utils.django_compat.is_installed("%s") only checks settings.INSTALLED_APPS, Apps aren't loaded yet.''', appname)
            return appname in settings.INSTALLED_APPS

    def get_app_names():
        return [
            appconfig.name for appconfig in apps.get_app_configs()
        ]

    def get_app_label(module):
        # Django 1.7 knows the "apps", so it can provide the meta information
        app_config = apps.get_containing_app_config(module)
        return app_config.label

    def get_meta_model_name(opts):
        # This name was available as of Django 1.6, mandatory in 1.8
        return opts.model_name



import django
from django.db import transaction


# New transaction support in Django 1.6
try:
    transaction_atomic = transaction.atomic
except AttributeError:
    transaction_atomic = transaction.commit_on_success


# Preserving admin form filters when adding parameters to the URL
try:
    # Django 1.6 supports this, and django-parler also applies this fix.
    from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
except ImportError:
    # Django <1.6 does not preserve filters
    def add_preserved_filters(context, form_url):
        return form_url

if django.VERSION >= (1,6):
    def is_queryset_empty(queryset):
        """
        Check whether a queryset is empty by using ``.none()``.
        """
        return queryset.query.is_empty()
else:
    from django.db.models.query import EmptyQuerySet
    def is_queryset_empty(queryset):
        return isinstance(queryset, EmptyQuerySet)



# Support for custom User models in Django 1.5+
from django.conf import settings


try:
    from django.contrib.auth import get_user_model
    AUTH_USER_MODEL = settings.AUTH_USER_MODEL
except ImportError:
    # django < 1.5
    from django.contrib.auth.models import User

    def get_user_model():
        return User

    AUTH_USER_MODEL = '{0}.{1}'.format(User._meta.app_label, User._meta.object_name)



"""
Optional integration with django-any-urlfield
"""
from __future__ import absolute_import
from django.db import models
from fluent_utils.django_compat import is_installed

if is_installed('any_urlfield'):
    from any_urlfield.models import AnyUrlField as BaseUrlField
else:
    BaseUrlField = models.URLField


# subclassing here so South or Django migrations detect a single class.
class AnyUrlField(BaseUrlField):
    """
    A CharField that can either refer to a CMS page ID, or external URL.

    If *django-any-urlfield* is not installed, only regular URLs can be used.
    """
    def __init__(self, *args, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 300  # Standardize
        super(AnyUrlField, self).__init__(*args, **kwargs)

    def south_field_triple(self):
        # Masquerade as normal URLField, so the soft-dependency also exists in the migrations.
        from south.modelsinspector import introspector
        path = "{0}.{1}".format(models.URLField.__module__, models.URLField.__name__)
        args, kwargs = introspector(self)
        return (path, args, kwargs)

    def deconstruct(self):
        # For Django 1.7 migrations, masquerade as normal URLField too
        name, path, args, kwargs = super(AnyUrlField, self).deconstruct()
        path = "{0}.{1}".format(models.URLField.__module__, models.URLField.__name__)
        return name, path, args, kwargs
